/*
 * File: MovieDemo.java
 * --------------------
 * This program illustrates the use of the MovieClip class.
 */

import acm.gui.*;
import acm.program.*;
import acm.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * This program illustrates the use of the <code>MovieClip</code> class.
 */
public class MovieDemo extends GraphicsProgram implements ChangeListener {

	public void init() {
		add(createUserInterface(), SOUTH);
		addActionListeners();
		volumeSlider.addChangeListener(this);
		rateSlider.addChangeListener(this);
	}

	private JPanel createUserInterface() {
		volumeSlider = new JSlider(0, 100, 100);
		volumeSlider.setMajorTickSpacing(100);
		volumeSlider.setMinorTickSpacing(25);
		volumeSlider.setPaintTicks(true);
		rateSlider = new JSlider(-50, 50, 10);
		rateSlider.setMajorTickSpacing(50);
		rateSlider.setMinorTickSpacing(10);
		rateSlider.setPaintTicks(true);
		textField = new JTextField(TEXT_FIELD_SIZE);
		textField.addActionListener(this);
		textField.setActionCommand("Open");
		JPanel panel = new VPanel();
		JPanel row1 = new JPanel();
		JPanel row2 = new JPanel();
		JPanel row3 = new JPanel();
		row1.setLayout(new FlowLayout());
		row2.setLayout(new FlowLayout());
		row3.setLayout(new FlowLayout());
		row1.add(new JButton("1984"));
		row1.add(new JButton("Browse"));
		row1.add(new JButton("Start"));
		row1.add(new JButton("Loop"));
		row1.add(new JButton("Stop"));
		row1.add(new JButton("Reset"));
		row2.add(new JLabel((isAppletMode()) ? "URL" : "File/URL"));
		row2.add(textField);
		row2.add(new JButton("Open"));
		row3.add(new JLabel("Volume:"));
		row3.add(volumeSlider);
		row3.add(new JLabel("   Rate:"));
		row3.add(rateSlider);
		panel.add(row1);
		panel.add(row2);
		panel.add(row3);
		return panel;
	}

	public void actionPerformed(ActionEvent e) {
		String cmd = e.getActionCommand();
		if (cmd.equals("Browse")) {
			try {
				File file = chooseInputFile();
				if (file != null) openMovie(new MovieClip(file));
			} catch (Exception ex) {
				throw new ErrorException("Can't find that movie");
			}
		} else if (cmd.equals("Open")) {
			try {
				openMovie(new MovieClip(textField.getText()));
			} catch (Exception ex) {
				throw new ErrorException("Can't find that movie");
			}
		} else if (cmd.equals("1984")) {
			try {
				openMovie(new MovieClip(new URL(SAMPLE_MOVIE)));
			} catch (Exception ex) {
				throw new ErrorException("Can't find that movie");
			}
		} else if (cmd.equals("Start")) {
			if (movie != null) {
				movie.play();
			}
		} else if (cmd.equals("Loop")) {
			if (movie != null) {
				movie.loop();
			}
		} else if (cmd.equals("Stop")) {
			if (movie != null) movie.stop();
		} else if (cmd.equals("Reset")) {
			if (movie != null) movie.setFrameIndex(0);
		}
	}

	public void stateChanged(ChangeEvent e) {
		if (movie == null) return;
		Object source = e.getSource();
		if (source == volumeSlider && movie != null) {
			movie.setVolume(volumeSlider.getValue() / 100.0);
		} else if (source == rateSlider) {
			movie.setPlaybackRate(rateSlider.getValue() / 10.0);
		}
	}

	private void openMovie(MovieClip clip) {
		movie = clip;
		movie.setVolume(volumeSlider.getValue() / 100.0);
		Dimension size = movie.getPreferredSize();
		double x = (getWidth() - size.width) / 2.0;
		double y = (getHeight() - size.height) / 2.0;
		add(movie, x, y);
	}

/*
 * Chooses a file for input using a file dialog.  If the user
 * clicks the Cancel button, the method returns null.  If
 * the user clicks Open, the method returns the File object.
 */
	private File chooseInputFile() {
		JFileChooser chooser = new JFileChooser();
		File cwd = new File(System.getProperty("user.dir"));
		File dir = new File(cwd, "movies");
		if (!dir.isDirectory()) {
			File home = new File(System.getProperty("user.home"));
			dir = new File(home, "Movies");
			if (!dir.isDirectory()) dir = new File(home, "Desktop/Movies");
		}
		if (dir.isDirectory()) chooser.setCurrentDirectory(dir);
		chooser.setFileFilter(new FileChooserFilter("*.mov", "Movie files"));
		int result = chooser.showOpenDialog(this);
		if (result != JFileChooser.APPROVE_OPTION) return null;
		return chooser.getSelectedFile();
	}

/* Private constants */
	private static final String SAMPLE_MOVIE = "http://www.apple-history.com/movies/1984.mov";
	private static final int TEXT_FIELD_SIZE = 30;

/* Private instance variables */
	private JTextField textField;
	private JSlider volumeSlider;
	private JSlider rateSlider;
	private MovieClip movie;

/* Standard Java entry point */
/* This method can be eliminated in most Java environments */
	public static void main(String[] args) {
		new MovieDemo().start(args);
	}
}
